#ifndef __ASM_OFFSETS_H__
#define __ASM_OFFSETS_H__
/*
 * DO NOT MODIFY.
 *
 * This file was generated by Kbuild
 */

#define CPUINFO_x86 1 /* offsetof(struct cpuinfo_x86, x86) */
#define CPUINFO_x86_vendor 2 /* offsetof(struct cpuinfo_x86, x86_vendor) */
#define CPUINFO_x86_model 0 /* offsetof(struct cpuinfo_x86, x86_model) */
#define CPUINFO_x86_stepping 4 /* offsetof(struct cpuinfo_x86, x86_stepping) */
#define CPUINFO_cpuid_level 36 /* offsetof(struct cpuinfo_x86, cpuid_level) */
#define CPUINFO_x86_capability 40 /* offsetof(struct cpuinfo_x86, x86_capability) */
#define CPUINFO_x86_vendor_id 136 /* offsetof(struct cpuinfo_x86, x86_vendor_id) */

#define PT_EBX 0 /* offsetof(struct pt_regs, bx) */
#define PT_ECX 4 /* offsetof(struct pt_regs, cx) */
#define PT_EDX 8 /* offsetof(struct pt_regs, dx) */
#define PT_ESI 12 /* offsetof(struct pt_regs, si) */
#define PT_EDI 16 /* offsetof(struct pt_regs, di) */
#define PT_EBP 20 /* offsetof(struct pt_regs, bp) */
#define PT_EAX 24 /* offsetof(struct pt_regs, ax) */
#define PT_DS 28 /* offsetof(struct pt_regs, ds) */
#define PT_ES 32 /* offsetof(struct pt_regs, es) */
#define PT_FS 36 /* offsetof(struct pt_regs, fs) */
#define PT_GS 40 /* offsetof(struct pt_regs, gs) */
#define PT_ORIG_EAX 44 /* offsetof(struct pt_regs, orig_ax) */
#define PT_EIP 48 /* offsetof(struct pt_regs, ip) */
#define PT_CS 52 /* offsetof(struct pt_regs, cs) */
#define PT_EFLAGS 56 /* offsetof(struct pt_regs, flags) */
#define PT_OLDESP 60 /* offsetof(struct pt_regs, sp) */
#define PT_OLDSS 64 /* offsetof(struct pt_regs, ss) */

#define saved_context_gdt_desc 32 /* offsetof(struct saved_context, gdt_desc) */

#define TSS_entry2task_stack 8204 /* offsetof(struct cpu_entry_area, tss.x86_tss.sp1) - offsetofend(struct cpu_entry_area, entry_stack_page.stack) */

#define EFI_svam 40 /* offsetof(efi_runtime_services_t, set_virtual_address_map) */

#define TASK_threadsp 4252 /* offsetof(struct task_struct, thread.sp) */
#define TASK_stack_canary 1184 /* offsetof(struct task_struct, stack_canary) */

#define pbe_address 0 /* offsetof(struct pbe, address) */
#define pbe_orig_address 4 /* offsetof(struct pbe, orig_address) */
#define pbe_next 8 /* offsetof(struct pbe, next) */

#define IA32_SIGCONTEXT_ax 44 /* offsetof(struct sigcontext_32, ax) */
#define IA32_SIGCONTEXT_bx 32 /* offsetof(struct sigcontext_32, bx) */
#define IA32_SIGCONTEXT_cx 40 /* offsetof(struct sigcontext_32, cx) */
#define IA32_SIGCONTEXT_dx 36 /* offsetof(struct sigcontext_32, dx) */
#define IA32_SIGCONTEXT_si 20 /* offsetof(struct sigcontext_32, si) */
#define IA32_SIGCONTEXT_di 16 /* offsetof(struct sigcontext_32, di) */
#define IA32_SIGCONTEXT_bp 24 /* offsetof(struct sigcontext_32, bp) */
#define IA32_SIGCONTEXT_sp 28 /* offsetof(struct sigcontext_32, sp) */
#define IA32_SIGCONTEXT_ip 56 /* offsetof(struct sigcontext_32, ip) */

#define IA32_RT_SIGFRAME_sigcontext 164 /* offsetof(struct rt_sigframe, uc.uc_mcontext) */

#define TDX_MODULE_rcx 0 /* offsetof(struct tdx_module_args, rcx) */
#define TDX_MODULE_rdx 8 /* offsetof(struct tdx_module_args, rdx) */
#define TDX_MODULE_r8 16 /* offsetof(struct tdx_module_args, r8) */
#define TDX_MODULE_r9 24 /* offsetof(struct tdx_module_args, r9) */
#define TDX_MODULE_r10 32 /* offsetof(struct tdx_module_args, r10) */
#define TDX_MODULE_r11 40 /* offsetof(struct tdx_module_args, r11) */
#define TDX_MODULE_r12 48 /* offsetof(struct tdx_module_args, r12) */
#define TDX_MODULE_r13 56 /* offsetof(struct tdx_module_args, r13) */
#define TDX_MODULE_r14 64 /* offsetof(struct tdx_module_args, r14) */
#define TDX_MODULE_r15 72 /* offsetof(struct tdx_module_args, r15) */
#define TDX_MODULE_rbx 80 /* offsetof(struct tdx_module_args, rbx) */
#define TDX_MODULE_rdi 88 /* offsetof(struct tdx_module_args, rdi) */
#define TDX_MODULE_rsi 96 /* offsetof(struct tdx_module_args, rsi) */

#define BP_scratch 484 /* offsetof(struct boot_params, scratch) */
#define BP_secure_boot 492 /* offsetof(struct boot_params, secure_boot) */
#define BP_loadflags 529 /* offsetof(struct boot_params, hdr.loadflags) */
#define BP_hardware_subarch 572 /* offsetof(struct boot_params, hdr.hardware_subarch) */
#define BP_version 518 /* offsetof(struct boot_params, hdr.version) */
#define BP_kernel_alignment 560 /* offsetof(struct boot_params, hdr.kernel_alignment) */
#define BP_init_size 608 /* offsetof(struct boot_params, hdr.init_size) */
#define BP_pref_address 600 /* offsetof(struct boot_params, hdr.pref_address) */

#define PTREGS_SIZE 68 /* sizeof(struct pt_regs) */
#define TLB_STATE_user_pcid_flush_mask 14 /* offsetof(struct tlb_state, user_pcid_flush_mask) */
#define CPU_ENTRY_AREA_entry_stack 8192 /* offsetof(struct cpu_entry_area, entry_stack_page) */
#define SIZEOF_entry_stack 4096 /* sizeof(struct entry_stack) */
#define MASK_entry_stack -4096 /* (~(sizeof(struct entry_stack) - 1)) */
#define TSS_sp0 4 /* offsetof(struct tss_struct, x86_tss.sp0) */
#define TSS_sp1 12 /* offsetof(struct tss_struct, x86_tss.sp1) */
#define TSS_sp2 20 /* offsetof(struct tss_struct, x86_tss.sp2) */
#define X86_top_of_stack 12 /* offsetof(struct pcpu_hot, top_of_stack) */
#define X86_current_task 0 /* offsetof(struct pcpu_hot, current_task) */

#endif
