/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * PPC4xx Prototypes and definitions
 *
 * Copyright 2008 DENX Software Engineering, Stefan Roese <sr@denx.de>
 */

#ifndef __ASM_POWERPC_PPC4xx_H__
#define __ASM_POWERPC_PPC4xx_H__

extern void __noreturn ppc4xx_reset_system(char *cmd);

#endif /* __ASM_POWERPC_PPC4xx_H__ */
