#ifndef __ASM_OFFSETS_H__
#define __ASM_OFFSETS_H__
/*
 * DO NOT MODIFY.
 *
 * This file was generated by Kbuild
 */

#define THREAD 7296 /* offsetof(struct task_struct, thread) */
#define MM 1696 /* offsetof(struct task_struct, mm) */
#define TASK_CANARY 1832 /* offsetof(struct task_struct, stack_canary) */
#define PACA_CANARY 3192 /* offsetof(struct paca_struct, canary) */
#define TASK_STACK 264 /* offsetof(struct task_struct, stack) */
#define TASK_CPU 4 /* offsetof(struct task_struct, thread_info.cpu) */
#define KSP 0 /* offsetof(struct thread_struct, ksp) */
#define PT_REGS 16 /* offsetof(struct thread_struct, regs) */
#define THREAD_FPEXC_MODE 568 /* offsetof(struct thread_struct, fpexc_mode) */
#define THREAD_FPSTATE 32 /* offsetof(struct thread_struct, fp_state.fpr) */
#define THREAD_FPSAVEAREA 560 /* offsetof(struct thread_struct, fp_save_area) */
#define FPSTATE_FPSCR 512 /* offsetof(struct thread_fp_state, fpscr) */
#define THREAD_LOAD_FP 633 /* offsetof(struct thread_struct, load_fp) */
#define THREAD_VRSTATE 640 /* offsetof(struct thread_struct, vr_state.vr) */
#define THREAD_VRSAVEAREA 1168 /* offsetof(struct thread_struct, vr_save_area) */
#define THREAD_USED_VR 1184 /* offsetof(struct thread_struct, used_vr) */
#define VRSTATE_VSCR 512 /* offsetof(struct thread_vr_state, vscr) */
#define THREAD_LOAD_VEC 634 /* offsetof(struct thread_struct, load_vec) */
#define THREAD_USED_VSR 1188 /* offsetof(struct thread_struct, used_vsr) */
#define KSP_VSID 8 /* offsetof(struct thread_struct, ksp_vsid) */
#define PACATMSCRATCH 2368 /* offsetof(struct paca_struct, tm_scratch) */
#define THREAD_TM_TFHAR 1200 /* offsetof(struct thread_struct, tm_tfhar) */
#define THREAD_TM_TEXASR 1208 /* offsetof(struct thread_struct, tm_texasr) */
#define THREAD_TM_TFIAR 1216 /* offsetof(struct thread_struct, tm_tfiar) */
#define THREAD_TM_TAR 1608 /* offsetof(struct thread_struct, tm_tar) */
#define THREAD_TM_PPR 1616 /* offsetof(struct thread_struct, tm_ppr) */
#define THREAD_TM_DSCR 1624 /* offsetof(struct thread_struct, tm_dscr) */
#define THREAD_TM_AMR 1632 /* offsetof(struct thread_struct, tm_amr) */
#define PT_CKPT_REGS 1224 /* offsetof(struct thread_struct, ckpt_regs) */
#define THREAD_CKVRSTATE 2176 /* offsetof(struct thread_struct, ckvr_state.vr) */
#define THREAD_CKVRSAVE 2704 /* offsetof(struct thread_struct, ckvrsave) */
#define THREAD_CKFPSTATE 1648 /* offsetof(struct thread_struct, ckfp_state.fpr) */
#define TM_FRAME_SIZE 736 /* STACK_INT_FRAME_SIZE + 16 */
#define TI_LOCAL_FLAGS 8 /* offsetof(struct thread_info, local_flags) */
#define DCACHEL1BLOCKSIZE 8 /* offsetof(struct ppc64_caches, l1d.block_size) */
#define DCACHEL1LOGBLOCKSIZE 12 /* offsetof(struct ppc64_caches, l1d.log_block_size) */
#define PACAPACAINDEX 8 /* offsetof(struct paca_struct, paca_index) */
#define PACAPROCSTART 58 /* offsetof(struct paca_struct, cpu_start) */
#define PACAKSAVE 2320 /* offsetof(struct paca_struct, kstack) */
#define PACACURRENT 2312 /* offsetof(struct paca_struct, __current) */
#define PACA_THREAD_INFO 2312 /* offsetof(struct paca_struct, __current) + offsetof(struct task_struct, thread_info) */
#define PACASAVEDMSR 2336 /* offsetof(struct paca_struct, saved_msr) */
#define PACAR1 2328 /* offsetof(struct paca_struct, saved_r1) */
#define PACATOC 16 /* offsetof(struct paca_struct, kernel_toc) */
#define PACAKBASE 24 /* offsetof(struct paca_struct, kernelbase) */
#define PACAKMSR 32 /* offsetof(struct paca_struct, kernel_msr) */
#define PACAHSRR_VALID 2352 /* offsetof(struct paca_struct, hsrr_valid) */
#define PACASRR_VALID 2353 /* offsetof(struct paca_struct, srr_valid) */
#define PACAIRQSOFTMASK 2354 /* offsetof(struct paca_struct, irq_soft_mask) */
#define PACAIRQHAPPENED 2355 /* offsetof(struct paca_struct, irq_happened) */
#define PACA_FTRACE_ENABLED 2596 /* offsetof(struct paca_struct, ftrace_enabled) */
#define PACA_EXGEN 128 /* offsetof(struct paca_struct, exgen) */
#define PACA_EXMC 2488 /* offsetof(struct paca_struct, exmc) */
#define PACA_EXNMI 2408 /* offsetof(struct paca_struct, exnmi) */
#define PACA_SLBSHADOWPTR 64 /* offsetof(struct paca_struct, slb_shadow_ptr) */
#define SLBSHADOW_STACKVSID 40 /* offsetof(struct slb_shadow, save_area[2 - 1].vsid) */
#define SLBSHADOW_STACKESID 32 /* offsetof(struct slb_shadow, save_area[2 - 1].esid) */
#define SLBSHADOW_SAVEAREA 16 /* offsetof(struct slb_shadow, save_area) */
#define LPPACA_PMCINUSE 187 /* offsetof(struct lppaca, pmcregs_in_use) */
#define PACA_PMCINUSE 2357 /* offsetof(struct paca_struct, pmcregs_in_use) */
#define LPPACA_YIELDCOUNT 256 /* offsetof(struct lppaca, yield_count) */
#define PACAEMERGSP 40 /* offsetof(struct paca_struct, emergency_sp) */
#define PACAMCEMERGSP 2576 /* offsetof(struct paca_struct, mc_emergency_sp) */
#define PACA_NMI_EMERG_SP 2568 /* offsetof(struct paca_struct, nmi_emergency_sp) */
#define PACA_IN_MCE 2586 /* offsetof(struct paca_struct, in_mce) */
#define PACA_IN_NMI 2584 /* offsetof(struct paca_struct, in_nmi) */
#define PACA_RFI_FLUSH_FALLBACK_AREA 3152 /* offsetof(struct paca_struct, rfi_flush_fallback_area) */
#define PACA_EXRFI 3072 /* offsetof(struct paca_struct, exrfi) */
#define PACA_L1D_FLUSH_SIZE 3160 /* offsetof(struct paca_struct, l1d_flush_size) */
#define PACAHWCPUID 56 /* offsetof(struct paca_struct, hw_cpu_id) */
#define PACAKEXECSTATE 59 /* offsetof(struct paca_struct, kexec_state) */
#define PACA_DSCR_DEFAULT 88 /* offsetof(struct paca_struct, dscr_default) */
#define PACA_EXIT_SAVE_R1 2344 /* offsetof(struct paca_struct, exit_save_r1) */
#define PACA_SPRG_VDSO 2360 /* offsetof(struct paca_struct, sprg_vdso) */
#define RTASBASE 8 /* offsetof(struct rtas_t, base) */
#define RTASENTRY 0 /* offsetof(struct rtas_t, entry) */
#define INT_FRAME_SIZE 720 /* STACK_INT_FRAME_SIZE */
#define SWITCH_FRAME_SIZE 432 /* STACK_SWITCH_FRAME_SIZE */
#define GPR0 48 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[0]) */
#define GPR1 56 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[1]) */
#define GPR2 64 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[2]) */
#define GPR3 72 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[3]) */
#define GPR4 80 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[4]) */
#define GPR5 88 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[5]) */
#define GPR6 96 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[6]) */
#define GPR7 104 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[7]) */
#define GPR8 112 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[8]) */
#define GPR9 120 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[9]) */
#define GPR10 128 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[10]) */
#define GPR11 136 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[11]) */
#define GPR12 144 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[12]) */
#define GPR13 152 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, gpr[13]) */
#define _NIP 304 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, nip) */
#define _MSR 312 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, msr) */
#define _CTR 328 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, ctr) */
#define _LINK 336 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, link) */
#define _CCR 352 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, ccr) */
#define _XER 344 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, xer) */
#define _DAR 376 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, dar) */
#define _DEAR 376 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, dear) */
#define _DSISR 384 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, dsisr) */
#define _ESR 384 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, esr) */
#define ORIG_GPR3 320 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, orig_gpr3) */
#define RESULT 392 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, result) */
#define _TRAP 368 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, trap) */
#define SOFTE 360 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, softe) */
#define _PPR 400 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, ppr) */
#define STACK_REGS_AMR 416 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, amr) */
#define STACK_REGS_IAMR 424 /* STACK_INT_FRAME_REGS + offsetof(struct pt_regs, iamr) */
#define CPU_SPEC_FEATURES 16 /* offsetof(struct cpu_spec, cpu_features) */
#define CPU_SPEC_SETUP 64 /* offsetof(struct cpu_spec, cpu_setup) */
#define CPU_SPEC_RESTORE 72 /* offsetof(struct cpu_spec, cpu_restore) */
#define pbe_address 0 /* offsetof(struct pbe, address) */
#define pbe_orig_address 8 /* offsetof(struct pbe, orig_address) */
#define pbe_next 16 /* offsetof(struct pbe, next) */
#define VDSO_DATA_OFFSET 248 /* offsetof(struct vdso_arch_data, data) */
#define VDSO_RNG_DATA_OFFSET 728 /* offsetof(struct vdso_arch_data, rng_data) */
#define CFG_TB_TICKS_PER_SEC 56 /* offsetof(struct vdso_arch_data, tb_ticks_per_sec) */
#define CFG_ICACHE_BLOCKSZ 116 /* offsetof(struct vdso_arch_data, icache_block_size) */
#define CFG_DCACHE_BLOCKSZ 112 /* offsetof(struct vdso_arch_data, dcache_block_size) */
#define CFG_ICACHE_LOGBLOCKSZ 124 /* offsetof(struct vdso_arch_data, icache_log_block_size) */
#define CFG_DCACHE_LOGBLOCKSZ 120 /* offsetof(struct vdso_arch_data, dcache_log_block_size) */
#define CFG_SYSCALL_MAP64 128 /* offsetof(struct vdso_arch_data, syscall_map) */
#define CFG_SYSCALL_MAP32 188 /* offsetof(struct vdso_arch_data, compat_syscall_map) */
#define VDSO_CLOCKMODE_OFFSET 252 /* offsetof(struct vdso_arch_data, data[0].clock_mode) */
#define VDSO_CLOCKMODE_TIMENS 2147483647 /* VDSO_CLOCKMODE_TIMENS */
#define BUG_ENTRY_SIZE 12 /* sizeof(struct bug_entry) */
#define VCPU_HOST_STACK 512 /* offsetof(struct kvm_vcpu, arch.host_stack) */
#define VCPU_HOST_PID 520 /* offsetof(struct kvm_vcpu, arch.host_pid) */
#define VCPU_GUEST_PID 5844 /* offsetof(struct kvm_vcpu, arch.pid) */
#define VCPU_GPRS 3584 /* offsetof(struct kvm_vcpu, arch.regs.gpr) */
#define VCPU_VRSAVE 5448 /* offsetof(struct kvm_vcpu, arch.vrsave) */
#define VCPU_FPRS 3968 /* offsetof(struct kvm_vcpu, arch.fp.fpr) */
#define VCPU_VRS 4496 /* offsetof(struct kvm_vcpu, arch.vr.vr) */
#define VCPU_XER 3880 /* offsetof(struct kvm_vcpu, arch.regs.xer) */
#define VCPU_CTR 3864 /* offsetof(struct kvm_vcpu, arch.regs.ctr) */
#define VCPU_LR 3872 /* offsetof(struct kvm_vcpu, arch.regs.link) */
#define VCPU_TAR 5152 /* offsetof(struct kvm_vcpu, arch.tar) */
#define VCPU_CR 3888 /* offsetof(struct kvm_vcpu, arch.regs.ccr) */
#define VCPU_PC 3840 /* offsetof(struct kvm_vcpu, arch.regs.nip) */
#define VCPU_MSR 7936 /* offsetof(struct kvm_vcpu, arch.shregs.msr) */
#define VCPU_SRR0 7912 /* offsetof(struct kvm_vcpu, arch.shregs.srr0) */
#define VCPU_SRR1 7920 /* offsetof(struct kvm_vcpu, arch.shregs.srr1) */
#define VCPU_SPRG0 7880 /* offsetof(struct kvm_vcpu, arch.shregs.sprg0) */
#define VCPU_SPRG1 7888 /* offsetof(struct kvm_vcpu, arch.shregs.sprg1) */
#define VCPU_SPRG2 7896 /* offsetof(struct kvm_vcpu, arch.shregs.sprg2) */
#define VCPU_SPRG3 7904 /* offsetof(struct kvm_vcpu, arch.shregs.sprg3) */
#define VCPU_SHARED_SPRG3 56 /* offsetof(struct kvm_vcpu_arch_shared, sprg3) */
#define VCPU_SHARED_SPRG4 208 /* offsetof(struct kvm_vcpu_arch_shared, sprg4) */
#define VCPU_SHARED_SPRG5 216 /* offsetof(struct kvm_vcpu_arch_shared, sprg5) */
#define VCPU_SHARED_SPRG6 224 /* offsetof(struct kvm_vcpu_arch_shared, sprg6) */
#define VCPU_SHARED_SPRG7 232 /* offsetof(struct kvm_vcpu_arch_shared, sprg7) */
#define VCPU_SHADOW_PID 5836 /* offsetof(struct kvm_vcpu, arch.shadow_pid) */
#define VCPU_SHADOW_PID1 5840 /* offsetof(struct kvm_vcpu, arch.shadow_pid1) */
#define VCPU_SHARED 7752 /* offsetof(struct kvm_vcpu, arch.shared) */
#define VCPU_SHARED_MSR 88 /* offsetof(struct kvm_vcpu_arch_shared, msr) */
#define VCPU_SHADOW_MSR 5456 /* offsetof(struct kvm_vcpu, arch.shadow_msr) */
#define VCPU_SHARED_MAS0 168 /* offsetof(struct kvm_vcpu_arch_shared, mas0) */
#define VCPU_SHARED_MAS1 172 /* offsetof(struct kvm_vcpu_arch_shared, mas1) */
#define VCPU_SHARED_MAS2 184 /* offsetof(struct kvm_vcpu_arch_shared, mas2) */
#define VCPU_SHARED_MAS7_3 176 /* offsetof(struct kvm_vcpu_arch_shared, mas7_3) */
#define VCPU_SHARED_MAS4 192 /* offsetof(struct kvm_vcpu_arch_shared, mas4) */
#define VCPU_SHARED_MAS6 196 /* offsetof(struct kvm_vcpu_arch_shared, mas6) */
#define VCPU_KVM 0 /* offsetof(struct kvm_vcpu, kvm) */
#define KVM_LPID 2520 /* offsetof(struct kvm, arch.lpid) */
#define KVM_SDR1 2592 /* offsetof(struct kvm, arch.sdr1) */
#define KVM_HOST_LPID 2576 /* offsetof(struct kvm, arch.host_lpid) */
#define KVM_HOST_LPCR 2584 /* offsetof(struct kvm, arch.host_lpcr) */
#define KVM_HOST_SDR1 2600 /* offsetof(struct kvm, arch.host_sdr1) */
#define KVM_ENABLED_HCALLS 2800 /* offsetof(struct kvm, arch.enabled_hcalls) */
#define KVM_VRMA_SLB_V 2616 /* offsetof(struct kvm, arch.vrma_slb_v) */
#define KVM_SECURE_GUEST 2706 /* offsetof(struct kvm, arch.secure_guest) */
#define VCPU_DSISR 7944 /* offsetof(struct kvm_vcpu, arch.shregs.dsisr) */
#define VCPU_DAR 7928 /* offsetof(struct kvm_vcpu, arch.shregs.dar) */
#define VCPU_VPA 8416 /* offsetof(struct kvm_vcpu, arch.vpa.pinned_addr) */
#define VCPU_VPA_DIRTY 8449 /* offsetof(struct kvm_vcpu, arch.vpa.dirty) */
#define VCPU_HEIR 8600 /* offsetof(struct kvm_vcpu, arch.emul_inst) */
#define VCPU_CPU 32 /* offsetof(struct kvm_vcpu, cpu) */
#define VCPU_THREAD_CPU 7640 /* offsetof(struct kvm_vcpu, arch.thread_cpu) */
#define VCPU_PURR 5176 /* offsetof(struct kvm_vcpu, arch.purr) */
#define VCPU_SPURR 5184 /* offsetof(struct kvm_vcpu, arch.spurr) */
#define VCPU_IC 5192 /* offsetof(struct kvm_vcpu, arch.ic) */
#define VCPU_DSCR 5200 /* offsetof(struct kvm_vcpu, arch.dscr) */
#define VCPU_AMR 5208 /* offsetof(struct kvm_vcpu, arch.amr) */
#define VCPU_UAMOR 5216 /* offsetof(struct kvm_vcpu, arch.uamor) */
#define VCPU_IAMR 5224 /* offsetof(struct kvm_vcpu, arch.iamr) */
#define VCPU_CTRL 5232 /* offsetof(struct kvm_vcpu, arch.ctrl) */
#define VCPU_DABR 5240 /* offsetof(struct kvm_vcpu, arch.dabr) */
#define VCPU_DABRX 5236 /* offsetof(struct kvm_vcpu, arch.dabrx) */
#define VCPU_DAWR0 5248 /* offsetof(struct kvm_vcpu, arch.dawr0) */
#define VCPU_DAWRX0 5256 /* offsetof(struct kvm_vcpu, arch.dawrx0) */
#define VCPU_CIABR 5304 /* offsetof(struct kvm_vcpu, arch.ciabr) */
#define VCPU_HFLAGS 5160 /* offsetof(struct kvm_vcpu, arch.hflags) */
#define VCPU_DEC_EXPIRES 7568 /* offsetof(struct kvm_vcpu, arch.dec_expires) */
#define VCPU_PENDING_EXC 7576 /* offsetof(struct kvm_vcpu, arch.pending_exceptions) */
#define VCPU_CEDED 7584 /* offsetof(struct kvm_vcpu, arch.ceded) */
#define VCPU_PRODDED 7585 /* offsetof(struct kvm_vcpu, arch.prodded) */
#define VCPU_MMCR 5864 /* offsetof(struct kvm_vcpu, arch.mmcr) */
#define VCPU_MMCRA 5896 /* offsetof(struct kvm_vcpu, arch.mmcra) */
#define VCPU_MMCRS 5904 /* offsetof(struct kvm_vcpu, arch.mmcrs) */
#define VCPU_PMC 5912 /* offsetof(struct kvm_vcpu, arch.pmc) */
#define VCPU_SIAR 5952 /* offsetof(struct kvm_vcpu, arch.siar) */
#define VCPU_SDAR 5960 /* offsetof(struct kvm_vcpu, arch.sdar) */
#define VCPU_SIER 5968 /* offsetof(struct kvm_vcpu, arch.sier) */
#define VCPU_SLB 528 /* offsetof(struct kvm_vcpu, arch.slb) */
#define VCPU_SLB_MAX 3088 /* offsetof(struct kvm_vcpu, arch.slb_max) */
#define VCPU_SLB_NR 3092 /* offsetof(struct kvm_vcpu, arch.slb_nr) */
#define VCPU_FAULT_DSISR 7424 /* offsetof(struct kvm_vcpu, arch.fault_dsisr) */
#define VCPU_FAULT_DAR 7416 /* offsetof(struct kvm_vcpu, arch.fault_dar) */
#define VCPU_INTR_MSR 7432 /* offsetof(struct kvm_vcpu, arch.intr_msr) */
#define VCPU_LAST_INST 7592 /* offsetof(struct kvm_vcpu, arch.last_inst) */
#define VCPU_TRAP 7628 /* offsetof(struct kvm_vcpu, arch.trap) */
#define VCPU_CFAR 5312 /* offsetof(struct kvm_vcpu, arch.cfar) */
#define VCPU_PPR 5320 /* offsetof(struct kvm_vcpu, arch.ppr) */
#define VCPU_FSCR 5336 /* offsetof(struct kvm_vcpu, arch.fscr) */
#define VCPU_PSPB 5328 /* offsetof(struct kvm_vcpu, arch.pspb) */
#define VCPU_EBBHR 5352 /* offsetof(struct kvm_vcpu, arch.ebbhr) */
#define VCPU_EBBRR 5360 /* offsetof(struct kvm_vcpu, arch.ebbrr) */
#define VCPU_BESCR 5368 /* offsetof(struct kvm_vcpu, arch.bescr) */
#define VCPU_CSIGR 5376 /* offsetof(struct kvm_vcpu, arch.csigr) */
#define VCPU_TACR 5384 /* offsetof(struct kvm_vcpu, arch.tacr) */
#define VCPU_TCSCR 5392 /* offsetof(struct kvm_vcpu, arch.tcscr) */
#define VCPU_ACOP 5400 /* offsetof(struct kvm_vcpu, arch.acop) */
#define VCPU_WORT 5408 /* offsetof(struct kvm_vcpu, arch.wort) */
#define VCPU_HFSCR 5432 /* offsetof(struct kvm_vcpu, arch.hfscr) */
#define VCORE_ENTRY_EXIT 8 /* offsetof(struct kvmppc_vcore, entry_exit_map) */
#define VCORE_IN_GUEST 25 /* offsetof(struct kvmppc_vcore, in_guest) */
#define VCORE_NAPPING_THREADS 12 /* offsetof(struct kvmppc_vcore, napping_threads) */
#define VCORE_KVM 160 /* offsetof(struct kvmppc_vcore, kvm) */
#define VCORE_TB_OFFSET 168 /* offsetof(struct kvmppc_vcore, tb_offset) */
#define VCORE_TB_OFFSET_APPL 176 /* offsetof(struct kvmppc_vcore, tb_offset_applied) */
#define VCORE_LPCR 184 /* offsetof(struct kvmppc_vcore, lpcr) */
#define VCORE_PCR 200 /* offsetof(struct kvmppc_vcore, pcr) */
#define VCORE_DPDES 208 /* offsetof(struct kvmppc_vcore, dpdes) */
#define VCORE_VTB 216 /* offsetof(struct kvmppc_vcore, vtb) */
#define VCPU_SLB_E 16 /* offsetof(struct kvmppc_slb, orige) */
#define VCPU_SLB_V 24 /* offsetof(struct kvmppc_slb, origv) */
#define VCPU_SLB_SIZE 40 /* sizeof(struct kvmppc_slb) */
#define VCPU_TFHAR 5992 /* offsetof(struct kvm_vcpu, arch.tfhar) */
#define VCPU_TFIAR 6008 /* offsetof(struct kvm_vcpu, arch.tfiar) */
#define VCPU_TEXASR 6000 /* offsetof(struct kvm_vcpu, arch.texasr) */
#define VCPU_ORIG_TEXASR 6016 /* offsetof(struct kvm_vcpu, arch.orig_texasr) */
#define VCPU_GPR_TM 6088 /* offsetof(struct kvm_vcpu, arch.gpr_tm) */
#define VCPU_FPRS_TM 6352 /* offsetof(struct kvm_vcpu, arch.fp_tm.fpr) */
#define VCPU_VRS_TM 6880 /* offsetof(struct kvm_vcpu, arch.vr_tm.vr) */
#define VCPU_VRSAVE_TM 7408 /* offsetof(struct kvm_vcpu, arch.vrsave_tm) */
#define VCPU_CR_TM 6024 /* offsetof(struct kvm_vcpu, arch.cr_tm) */
#define VCPU_XER_TM 6032 /* offsetof(struct kvm_vcpu, arch.xer_tm) */
#define VCPU_LR_TM 6040 /* offsetof(struct kvm_vcpu, arch.lr_tm) */
#define VCPU_CTR_TM 6048 /* offsetof(struct kvm_vcpu, arch.ctr_tm) */
#define VCPU_AMR_TM 6056 /* offsetof(struct kvm_vcpu, arch.amr_tm) */
#define VCPU_PPR_TM 6064 /* offsetof(struct kvm_vcpu, arch.ppr_tm) */
#define VCPU_DSCR_TM 6072 /* offsetof(struct kvm_vcpu, arch.dscr_tm) */
#define VCPU_TAR_TM 6080 /* offsetof(struct kvm_vcpu, arch.tar_tm) */
#define HSTATE_HOST_R1 2688 /* offsetof(struct paca_struct, kvm_hstate.host_r1) */
#define HSTATE_HOST_R2 2696 /* offsetof(struct paca_struct, kvm_hstate.host_r2) */
#define HSTATE_HOST_MSR 2704 /* offsetof(struct paca_struct, kvm_hstate.host_msr) */
#define HSTATE_VMHANDLER 2712 /* offsetof(struct paca_struct, kvm_hstate.vmhandler) */
#define HSTATE_SCRATCH0 2720 /* offsetof(struct paca_struct, kvm_hstate.scratch0) */
#define HSTATE_SCRATCH1 2728 /* offsetof(struct paca_struct, kvm_hstate.scratch1) */
#define HSTATE_SCRATCH2 2736 /* offsetof(struct paca_struct, kvm_hstate.scratch2) */
#define HSTATE_IN_GUEST 2744 /* offsetof(struct paca_struct, kvm_hstate.in_guest) */
#define HSTATE_RESTORE_HID5 2745 /* offsetof(struct paca_struct, kvm_hstate.restore_hid5) */
#define HSTATE_NAPPING 2746 /* offsetof(struct paca_struct, kvm_hstate.napping) */
#define HSTATE_HWTHREAD_REQ 2747 /* offsetof(struct paca_struct, kvm_hstate.hwthread_req) */
#define HSTATE_HWTHREAD_STATE 2748 /* offsetof(struct paca_struct, kvm_hstate.hwthread_state) */
#define HSTATE_KVM_VCPU 2752 /* offsetof(struct paca_struct, kvm_hstate.kvm_vcpu) */
#define HSTATE_KVM_VCORE 2760 /* offsetof(struct paca_struct, kvm_hstate.kvm_vcore) */
#define HSTATE_HOST_IPI 2749 /* offsetof(struct paca_struct, kvm_hstate.host_ipi) */
#define HSTATE_PTID 2750 /* offsetof(struct paca_struct, kvm_hstate.ptid) */
#define HSTATE_FAKE_SUSPEND 2751 /* offsetof(struct paca_struct, kvm_hstate.fake_suspend) */
#define HSTATE_MMCR0 2808 /* offsetof(struct paca_struct, kvm_hstate.host_mmcr[0]) */
#define HSTATE_MMCR1 2816 /* offsetof(struct paca_struct, kvm_hstate.host_mmcr[1]) */
#define HSTATE_MMCRA 2824 /* offsetof(struct paca_struct, kvm_hstate.host_mmcr[2]) */
#define HSTATE_SIAR 2832 /* offsetof(struct paca_struct, kvm_hstate.host_mmcr[3]) */
#define HSTATE_SDAR 2840 /* offsetof(struct paca_struct, kvm_hstate.host_mmcr[4]) */
#define HSTATE_MMCR2 2848 /* offsetof(struct paca_struct, kvm_hstate.host_mmcr[5]) */
#define HSTATE_SIER 2856 /* offsetof(struct paca_struct, kvm_hstate.host_mmcr[6]) */
#define HSTATE_PMC1 2864 /* offsetof(struct paca_struct, kvm_hstate.host_pmc[0]) */
#define HSTATE_PMC2 2868 /* offsetof(struct paca_struct, kvm_hstate.host_pmc[1]) */
#define HSTATE_PMC3 2872 /* offsetof(struct paca_struct, kvm_hstate.host_pmc[2]) */
#define HSTATE_PMC4 2876 /* offsetof(struct paca_struct, kvm_hstate.host_pmc[3]) */
#define HSTATE_PMC5 2880 /* offsetof(struct paca_struct, kvm_hstate.host_pmc[4]) */
#define HSTATE_PMC6 2884 /* offsetof(struct paca_struct, kvm_hstate.host_pmc[5]) */
#define HSTATE_PURR 2896 /* offsetof(struct paca_struct, kvm_hstate.host_purr) */
#define HSTATE_SPURR 2904 /* offsetof(struct paca_struct, kvm_hstate.host_spurr) */
#define HSTATE_DSCR 2912 /* offsetof(struct paca_struct, kvm_hstate.host_dscr) */
#define HSTATE_DABR 2800 /* offsetof(struct paca_struct, kvm_hstate.dabr) */
#define HSTATE_DECEXP 2920 /* offsetof(struct paca_struct, kvm_hstate.dec_expires) */
#define HSTATE_SPLIT_MODE 2928 /* offsetof(struct paca_struct, kvm_hstate.kvm_split_mode) */
#define IPI_PRIORITY 4 /* IPI_PRIORITY */
#define KVM_SPLIT_RPR 0 /* offsetof(struct kvm_split_mode, rpr) */
#define KVM_SPLIT_PMMAR 8 /* offsetof(struct kvm_split_mode, pmmar) */
#define KVM_SPLIT_LDBAR 16 /* offsetof(struct kvm_split_mode, ldbar) */
#define KVM_SPLIT_DO_NAP 25 /* offsetof(struct kvm_split_mode, do_nap) */
#define KVM_SPLIT_NAPPED 26 /* offsetof(struct kvm_split_mode, napped) */
#define HSTATE_CFAR 2936 /* offsetof(struct paca_struct, kvm_hstate.cfar) */
#define HSTATE_PPR 2944 /* offsetof(struct paca_struct, kvm_hstate.ppr) */
#define HSTATE_HOST_FSCR 2952 /* offsetof(struct paca_struct, kvm_hstate.host_fscr) */
#define KVM_MAGIC_SCRATCH1 0 /* offsetof(struct kvm_vcpu_arch_shared, scratch1) */
#define KVM_MAGIC_SCRATCH2 8 /* offsetof(struct kvm_vcpu_arch_shared, scratch2) */
#define KVM_MAGIC_SCRATCH3 16 /* offsetof(struct kvm_vcpu_arch_shared, scratch3) */
#define KVM_MAGIC_INT 100 /* offsetof(struct kvm_vcpu_arch_shared, int_pending) */
#define KVM_MAGIC_MSR 88 /* offsetof(struct kvm_vcpu_arch_shared, msr) */
#define KVM_MAGIC_CRITICAL 24 /* offsetof(struct kvm_vcpu_arch_shared, critical) */
#define KVM_MAGIC_SR 104 /* offsetof(struct kvm_vcpu_arch_shared, sr) */
#define PPC_DBELL_SERVER 5 /* PPC_DBELL_SERVER */
#define BPT_SIZE 16 /* BPT_SIZE */

#endif
