/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * SPU info structures
 *
 * (C) Copyright 2006 IBM Corp.
 *
 * Author: Dwayne Grant McConnell <decimal@us.ibm.com>
 */
#ifndef _SPU_INFO_H
#define _SPU_INFO_H

#include <asm/spu.h>
#include <uapi/asm/spu_info.h>

#endif
