/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _ASM_POWERPC_BOOK3S_TLBFLUSH_H
#define _ASM_POWERPC_BOOK3S_TLBFLUSH_H

#ifdef CONFIG_PPC64
#include <asm/book3s/64/tlbflush.h>
#else
#include <asm/book3s/32/tlbflush.h>
#endif

#endif /* _ASM_POWERPC_BOOK3S_TLBFLUSH_H */
