/* SPDX-License-Identifier: GPL-2.0+ WITH Linux-syscall-note */
/*
 * This file contains the system call numbers.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */
#ifndef _UAPI_ASM_POWERPC_UNISTD_H_
#define _UAPI_ASM_POWERPC_UNISTD_H_

#ifndef __powerpc64__
#include <asm/unistd_32.h>
#else
#include <asm/unistd_64.h>
#endif

#endif /* _UAPI_ASM_POWERPC_UNISTD_H_ */
