#define vdso64_offset_ftr_fixup_end	0x016c0
#define vdso64_offset_ftr_fixup_start	0x01570
#define vdso64_offset_fw_ftr_fixup_end	0x016c0
#define vdso64_offset_fw_ftr_fixup_start	0x016c0
#define vdso64_offset_lwsync_fixup_end	0x016c0
#define vdso64_offset_lwsync_fixup_start	0x016c0
#define vdso64_offset_mmu_ftr_fixup_end	0x016c0
#define vdso64_offset_mmu_ftr_fixup_start	0x016c0
#define vdso64_offset_sigtramp_rt64	0x0480
