#define UTS_MACHINE		"aarch64"
#define LINUX_COMPILE_BY	"root"
#define LINUX_COMPILE_HOST	"dola.arm.dev.gentoo.org"
#define LINUX_COMPILER		"aarch64-unknown-linux-gnu-gcc (Gentoo 15.2.1_p20260214 p5) 15.2.1 20260214, GNU ld (Gentoo 2.46.0 p1) 2.46.0"
